# Helper functions
function get_repository_name() {
local path=$1

	basename "$path"
}

function get_module_name() {
local path=$1

	basename "$path" | sed 's/\(.*\)\.\([^.]*$\)/\1/'
}

function get_module_type() {
local path=$1

	basename "$path" | sed 's/\(.*\)\.\([^.]*$\)/\2/'
}

function is_repository() {
local path=$1

	local repository_name=$(get_repository_name $path)

	[ -d $path/.git -a -f $path/$repository_name.list ]
}

function is_module() {
local path=$1

	[ -d $path/.git ]
}

function find_module() {
local path=$1

	while ! is_module $path && [ $path != / ] ; do
		path=$(dirname $path)
	done

	if ! is_module $path ; then
		return
	fi

	echo $path/
}

function find_repository() {
local path=$1

	while ! is_repository "$path" && [ "$path" != / ] ; do
		path=$(dirname $path)
	done

	if ! is_repository $path ; then
		return
	fi

	echo $path/
}

function get_current_module_name() {
	get_module_name $(find_module $PWD)
}

function get_current_repository_name() {
	get_repository_name $(find_repository $PWD)
}

function assert_is_module() {
local path=$1

	if ! is_module $path ; then
		echo $(error "You need to be in a module directory!") >&2
		exit 1
	fi
}

function assert_is_repository() {
local path=$1

	if ! is_repository $path ; then
		echo $(error "You need to be in a repository directory!") >&2
		exit 1
	fi
}

function is_absolute_path() {
local path="$1"

	echo "$path" | grep '^/' 2>&1 >/dev/null
}

function get_absolute_path() {
local path="$1"
local reference="${2:-$PWD}"

	if is_absolute_path "$path" ; then
		echo "$path"
	else
		echo "$reference/$path"
	fi
}

