# Output filters for executed programs
function filter_apt() {
	sed "s/^E: /  \[${red}APT${normal}\]: ${red}/" | sed "s/\$/${normal}/"
}

# package functions
function is_debian_package_file() {
local name=$1

	echo $name | grep '\.deb$' 2>&1 >/dev/null
}

function get_debian_package_name() {
local package_file=$1

	basename $package_file | cut -d '_' -f 1
}

function get_debian_package_version() {
local package_file=$1

	dpkg-deb --field $package_file Version
}

function get_debian_package_file() {
local package=$1
local package_file=$2

	# Check if there's a package already downloaded...
	local deb_file=$(ls -1 /var/cache/apt/archives/${package}_*.deb  2>/dev/null | tail -1)

	if [ -z "$deb_file" ] ; then
		echo "  $(action Downloading) package $(module $package)..."

		# Try to download using APT
		# TODO: this should be doable without sudo using a cusom apt.conf setting
		#       special paths
		sudo apt-get --quiet 2 --download-only --reinstall install $package 2>&1 | filter_apt

		deb_file=$(ls -1 /var/cache/apt/archives/${package}_*.deb  2>/dev/null | tail -1)
	fi

	if [ -n "$deb_file" ] ; then
		cp "$deb_file" "$package_file"
	else
		echo "  $(error "No package available for $package")" >&2
		return 1
	fi
}

function extract_debian_package() {
local package_file=$1
local package_extract_dir=$2

	# Extract package into /tmp to get at the vendor configs
	mkdir -p $package_extract_dir
	dpkg-deb --extract $package_file $package_extract_dir
}

function import_debian_package() {
local package_file=$1

	local package_extract_dir=$(tmp_get_dir)

	extract_debian_package $package_file $package_extract_dir || return 1
	isisetup-module import $package_extract_dir/etc

	rm -rf $package_extract_dir
}

