function filter_git() {
	grep -v '^Committing initial tree ' |
	grep -v '^defaulting to local storage area$' |
	grep -v '^Branch already fully merged.$' |
	grep -v '^Merge ' |
	grep -v '^Already up-to-date.' |
	grep -v '^Tree change: ' |
	grep -v '^Trying simple merge with ' |
	grep -v '^Trying really trivial in-index merge...$' |
	grep -v '^Wonderful.$' |
	grep -v 'In-index merge$' |
	grep -v "^Hard links don't work - using copy" |
	grep -v "^error: duplicate parent" |
	grep -v '^$'
}

function filter_cogito() {
	grep -v "^A " |
	grep -v "^D " |
	grep -v "^M " |
	grep -v "^Cloned to " |
	grep -v '^cg-clone: .* already exists$' |
	grep -v "^Committed as " |
	grep -v "^Adding file " |
	grep -v "^Removing file " |
	grep -v '^Using hard links$' |
	grep -v '^Fetching head\.\.\.$' |
	grep -v '^Fetching objects\.\.\.$' |
	grep -v '^Fetching tags\.\.\.' |
	grep -v '^Applying changes...' |
	grep -v '^progress: ' |
	grep -v '^New branch: ' |
	grep -v '.* -> .*' |
	grep -v 'to .*\.\.\.' |
	grep -v 'Do cg-commit after reviewing the merge.' |
	grep -v 'cg-reset will cancel the merge.' |
	grep -v -i '^Warning: Suppressing default merge log messages in favour of the custom -m passed to me.' |
	grep -v '^cg-branch-ls: List of branches is empty. See cg-branch-add.' |
	grep -v '^Up to date.$' |
	grep -v '^Branch already merged in the working tree.$' |
	grep -v '^Refusing to make an empty commit - the tree was not modified$' |
	grep -v '^since the previous commit. If you really want to make the$' |
	grep -v '^commit, pass cg-commit the -f argument.$' |
	grep -v '^cg-branch-add: branch already exists$' |
	filter_git
}


# SCM abstration API
# ==================

# Initialize working directory
function init() {
local type="$1"
local name="$2"

	cg init -m "Created $type $name" 2>&1 | filter_cogito
}

# Clone working directory
function clone() {
local source=$1
local target=$2

	cg-clone $source $target 2>&1 | filter_cogito
}

# Commit outstanding changes
function commit() {
local message="$1"
local path="$2"

    if [ -n "$message" ] ; then
	    cg commit -q -m "$message" $path 2>&1 | filter_cogito
	else
	    cg commit -q $path
	fi
}

function log() {

	cg log
}

function add() {
local path="$1"

	git add "$path" 2>&1 | filter_git
}

function remove() {
local path="$1"

	cg rm -f "$path" 2>&1 | filter_cogito
}

function move() {
local source_path=$1
local destination_path=$2

	git mv $source_path $destination_path 2>&1 | filter_git
}

# Update all branches
function update() {
	branches=$(cg branch-ls 2>/dev/null | cut -f 1 | tr $'\n' ' ')

	local branch_count=$(echo $branches | wc -w)
	if [ $branch_count -gt 29 ] ; then
		for branch in $branches ; do
			cg update $branch 2>&1 | filter_cogito
		done
	else
		for branch in $branches ; do
			cg fetch $branch 2>&1 | filter_cogito
		done
	
		# Use git instead of Cogito as the latter lacks octopus merging
		if [ -n "$branches" ] ; then
			git-merge -n "Merged branches '$branches'" HEAD $branches 2>&1 | filter_git
		fi
	fi
}

function add_branch() {
local branch_name=$1
local branch_url=$2

	cg branch-add $branch_name $branch_url 2>&1 | filter_cogito
	cg fetch $branch_name 2>&1 | filter_cogito
	
	# Try to merge
	if [ -n "$(cg merge -c $branch_name 2>&1 | filter_cogito | grep 'unable to automatically determine merge base')" ]; then
		# If cg merge comlains about merge base try joining the branches
		cg merge -j -c $branch_name 2>&1 | filter_cogito
	fi

	commit "Added branch $branch_name"
}

